//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////
//
// Program        : Average
//
// Author         : Richard E. Pattis
//                  Computer Science Department
//                  University of California, Irvine
//                  Irvine, CA 92617-3435
//                  e-mail: pattis@ics.uci.edu
//
// Maintainer     : Author
//
//
// Description:
//
///   Average prompts the user for the name of a file that stores scores
// (all ints) from an exam. Then, it reads all the values stored in that
// file. The first value represents the maximum value for the test; the
// other values are the scores themselves. Finally, the last value is a
// sentinel, -1, that signals the end of the file. Again, all file values
// must be ints (otherwise an exception is thrown, the computation is
// abandoned, the exception is caught, and an error message is printed).
// In EBNF we would denote the form of the file as:
//   Maximum {Scores} -1
// Average computes and prints the average of all the scores in the file,
// excluding the first value (the maximum) and the sentinel (which are are
// in the file, but not actually scores). If there are no scores between
// these values (a legal possibility), Average prints a special error message
// rather than dividing by 0.
//
//   Note that I have over-commented this program, because it deals with
// a concept (reading data from files) that we have not covered in class.
// You will not need to change/write any file manipulating code. 
//
//
// Known Bugs     : None
//
// Future Plans   : Filter scores <0 or >Maximum; display the number filtered
//                  Compute and print the minimum and maximum scores
//
// Program History:
//    5/24/01: R. Pattis - Operational for 15-100
//    1/25/07: R. Pattis - Updated what is printed for exceptions
//
//////////////////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////////////////


import edu.uci.ics.pattis.introlib.Prompt;
import edu.uci.ics.pattis.introlib.TypedBufferReader;


public class Average {

  public static void main(String[] args) {
    try {

      //The declaration of inputFile is a reference to a newly constructed
      //TypedBufferReader object. This version of the constructor includes the
      //prompt String for the user. The constructor will continually reprompt
      //the user until he/she correctly enters the name of a file that is stored
      //on the computer. Then the code uses inputFile much like Prompt: e.g.,
      //x = inputFile.readInt() extracts the next int value from inputFile and
      //then stores that value into x (a variable of the primitive type int).

      TypedBufferReader inputFile = new TypedBufferReader("Enter name of file that contains exam scores");

      int scoreCount = 0;                 //Statistics, initialized to 0
      int scoreSum   = 0;	
      int examMax    =inputFile.readInt();//Initialized as the first value from inputFile


      //This for loop repeatedly reads scores from inputFile and updates various
      //statistics. A score of -1 (the sentinel) marks the end of the file and
      //causes the loop to terminate.

      for(;;) {
        int score = inputFile.readInt();  //Declare score, store next value from inputFile into it
        ////////////////
        if (score == -1)                  //Terminate when score stores the sentinel
          break;
        ////////////////

        scoreCount++;                     //score not the sentinel: update statistics
        scoreSum += score;
      }


      //Report "appropriate" statistics on all exam scores read from inputFile

      System.out.println();
      System.out.println("Read " + scoreCount+ " scores");
      System.out.println("The maximum allowable score was " + examMax);

      //Avoid "average" computation if no scores were read
      if (scoreCount == 0)
        System.out.println("No scores to average");
      else
        System.out.println("Average = " + (double)scoreSum/(double)scoreCount);


      //Because the main loop is controlled by a sentinel, we do not need to catch an
      //EndOfFileException to stop the loop. We will assume that there are no other
      //problems in the file causing exceptions, but if there are, we will catch ALL
      //exceptions here, and print an error message. Your program, on the data we are
      //testing it on, should never have to execute this code.
    }catch (Exception e) {
      System.out.println("Input file error: main method in Average class terminating");
      e.printStackTrace();
      System.exit(0);  
    }
  }
}
